@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2009 Jonathan Peirce
+# Copyright (C) 2010 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """
@@ -11,6 +11,6 @@ def rush(value=True):
     > nice python ......
     
     """
-    pass
+    return False
   
   
\ No newline at end of file
