@@ -1,13 +1,18 @@
 """Tools to help with calibrations
 """
 # Part of the PsychoPy library
-# Copyright (C) 2009 Jonathan Peirce
+# Copyright (C) 2010 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 from calibData import *
-from psychopy import __version__, log, serial
+from psychopy import __version__, log
 import time
-    
+
+try: 
+    import serial
+    haveSerial=True
+except:
+    haveSerial=False
 import string, os, time, glob, cPickle, sys
 from copy import deepcopy, copy
 
@@ -53,13 +58,12 @@ pr650code={'OK':'000\r\n',#this is returned after measure
     }
 
 def findPR650(ports=None):
-    """Try to find a COM port with a PR650 connected! Returns a Photometer object
-    e.g.::
-    
-        pr650 = findPR650([portNumber, anotherPortNumber])#tests specific ports only
-        pr650 = findPR650() #sweeps ports 0 to 10 searching for PR650
-        
+    """DEPRECATED (as of v.1.60.01). Use :func:`psychopy.hardware.findPhotometer()` instead, which
+    finds a wider range of devices   
     """
+    log.error("DEPRECATED (as of v.1.60.01). Use psychopy.hardware.findPhotometer() instead, which "\
+    +"finds a wider range of devices")
+    print 'here'
     if ports==None:
         if sys.platform=='darwin':
             ports=[]
@@ -83,164 +87,28 @@ def findPR650(ports=None):
             log.info(' ...OK\n'); log.console.flush()
             break
         else:
+            pr650=None
             log.info('...Nope!\n\t'); log.console.flush()
     return pr650
 
 class Photometer:
     """
-    A class to define a calibration device that connects to
-    a serial port. Currently it must be a PR650, but could
-    well be expanded in the future
-
-        ``myPR650 = Photometer(port, meterType="PR650")``
-
-    jwp (12/2003)
+    DEPRECATED (v.1.60.00):
+    
+    Import explicit flavour of photometer as needed e.g.::
+        
+        from psychopy.hardware.pr import PR650
+        from psychopy.hardware.minolta import LS100
+        
+    Or simply::
+        
+        from psychopy import hardware
+        photometer = hardware.findPhotometer()
+        
     """
     def __init__(self, port, meterType="PR650", verbose=True):
-        if type(port) in [int, float]:
-            self.portNumber = port #add one so that port 1=COM1
-            self.portString = 'COM%i' %self.portNumber#add one so that port 1=COM1
-        else:
-            self.portString = port
-            self.portNumber=None
-        self.type=meterType
-        self.isOpen=0
-        self.lastQual=0
-
-        try:
-            #try to open the actual port
-            if sys.platform =='darwin':
-                self.com = serial.Serial(self.portString)
-            elif sys.platform=='win32':
-                self.com = serial.Serial(self.portString)
-            else: log.error("I don't know how to handle serial ports on %s" %sys.platform)
-
-            self.com.setBaudrate(9600)
-            self.com.setParity('N')#none
-            self.com.setStopbits(1)
-            self.com.open()
-            self.isOpen=1
-            log.info("Successfully opened %s" %self.portString)
-            self.OK = True
-            time.sleep(1.0) #wait while establish connection
-            reply = self.sendMessage('b1\n')        #turn on the backlight as feedback
-        except:
-            if verbose: log.error("Couldn't open serial port %s" %self.portString)
-            self.OK = False
-            return None #NB the user still receives a photometer object but with .OK=False
-
-        #also check that the reply is what was expected
-        if reply != pr650code['OK']:
-            if verbose: log.error("pr650 isn't communicating")
-            self.OK = False
-        else:
-            reply = self.sendMessage('s01,,,,,,01,1')#set command to make sure using right units etc...
-
-
-    def sendMessage(self, message, timeout=0.5, DEBUG=False):
-        """
-        send a command to the photometer and wait an alloted
-        timeout for a response (Timeout should be long for low
-        light measurements)
-        """
-        if message[-1]!='\n': message+='\n'     #append a newline if necess
-
-        #flush the read buffer first
-        self.com.read(self.com.inWaiting())#read as many chars as are in the buffer
-
-        #send the message
-        self.com.write(message)
-        self.com.flush()
-            #time.sleep(0.1)  #PR650 gets upset if hurried!
-
-        #get feedback (within timeout limit)
-        self.com.setTimeout(timeout)
-        log.debug(message)#send complete message
-        if message in ['d5', 'd5\n']: #we need a spectrum which will have multiple lines
-            return self.com.readlines()
-        else:
-            return self.com.readline()
-
-
-    def measure(self, timeOut=30.0):
-        t1 = time.clock()
-        reply = self.sendMessage('m0\n', timeOut) #measure and hold data
-        #using the hold data method the PR650 we can get interogate it
-        #several times for a single measurement
-
-        if reply==pr650code['OK']:
-            raw = self.sendMessage('d2')
-            xyz = string.split(raw,',')#parse into words
-            self.lastQual = str(xyz[0])
-            if pr650code[self.lastQual]=='OK':
-                self.lastLum = float(xyz[3])
-            else: self.lastLum = 0.0
-        else:
-            log.warning("Didn't collect any data (extend timeout?)")
-
-    def getLastSpectrum(self, parse=True):
-        """This retrieves the spectrum from the last call to ``.measure()``
-
-        If ``parse=True`` (default):
-        The format is a num array with 100 rows [nm, power]
-
-        otherwise:
-        The output will be the raw string from the PR650 and should then
-        be passed to ``.parseSpectrumOutput()``. It's more
-        efficient to parse R,G,B strings at once than each individually.
-        """
-        raw = self.sendMessage('d5')#returns a list where each list
-        if parse:
-            return self.parseSpectrumOutput(raw[2:])#skip the first 2 entries (info)
-        else:
-            return raw
-
-    def parseSpectrumOutput(self, rawStr):
-        """Parses the strings from the PR650 as received after sending
-        the command 'd5'.
-        The input argument "rawStr" can be the output from a single
-        phosphor spectrum measurement or a list of 3 such measurements
-        [rawR, rawG, rawB].
-        """
-
-        if len(rawStr)==3:
-            RGB=True
-            rawR=rawStr[0][2:]
-            rawG=rawStr[1][2:]
-            rawB=rawStr[2][2:]
-            nPoints = len(rawR)
-        else:
-            RGB=False
-            nPoints=len(rawStr)
-            raw=raw[2:]
-
-        nm = []
-        if RGB:
-            power=[[],[],[]]
-            for n in range(nPoints):
-                #each entry in list is a string like this:
-                thisNm, thisR = string.split(rawR[n],',')
-                thisR = thisR.replace('\r\n','')
-                thisNm, thisG = string.split(rawG[n],',')
-                thisG = thisG.replace('\r\n','')
-                thisNm, thisB = string.split(rawB[n],',')
-                thisB = thisB.replace('\r\n','')
-                exec('nm.append(%s)' %thisNm)
-                exec('power[0].append(%s)' %thisR)
-                exec('power[1].append(%s)' %thisG)
-                exec('power[2].append(%s)' %thisB)
-                #if progDlg: progDlg.Update(n)
-        else:
-            power = []
-            for n, point in enumerate(rawStr):
-                #each entry in list is a string like this:
-                thisNm, thisPower = string.split(point,',')
-                nm.append(thisNm)
-                power.append(thisPower.replace('\r\n',''))
-            if progDlg: progDlg.Update(n)
-        #if progDlg: progDlg.Destroy()
-        return numpy.asarray(nm), numpy.asarray(power)
-
+        log.error(self.__doc__)
+        sys.exit()
 
 class Monitor:
     """Creates a monitor object for storing calibration details.
@@ -282,7 +150,7 @@ class Monitor:
         useBits=None,
         verbose=True,
         currentCalib={},
-    ):
+         ):
         """
         """
 
@@ -454,7 +322,7 @@ class Monitor:
             return self.currentCalib['spectraNM'], self.currentCalib['spectraRGB']
         else:
             return None, None
-    def getDKL_RGB(self, RECOMPUTE=True):
+    def getDKL_RGB(self, RECOMPUTE=False):
         """Returns the DKL->RGB conversion matrix.
         If one has been saved this will be returned.
         Otherwise, if power spectra are available for the
@@ -470,7 +338,7 @@ class Monitor:
         else:
             return self.currentCalib['dkl_rgb']
 
-    def getLMS_RGB(self, RECOMPUTE=True):
+    def getLMS_RGB(self, RECOMPUTE=False):
         """Returns the LMS->RGB conversion matrix.
         If one has been saved this will be returned.
         Otherwise (if power spectra are available for the
@@ -846,7 +714,7 @@ def makeLMS2RGB(nm,powerRGB):
     return cones_to_rgb
 
 
-def getLumSeriesPR650(lumLevels=8,
+def getLumSeries(lumLevels=8,
     winSize=(800,600),
     monitor=None,
     gamma=1.0,
@@ -854,36 +722,45 @@ def getLumSeriesPR650(lumLevels=8,
     useBits=False,
     autoMode='auto',
     stimSize = 0.3,
-    photometer='COM1'):
+    photometer=None):
     """
     Automatically measures a series of gun values and measures
-    the luminance with a PR650.
+    the luminance with a photometer.
 
     :Parameters:
-        photometer : is the number of the serial port PR650 is connected to, or
-            or Photometer object, from findPR650()
-        lumLevels : (=8) can be scalar (number of tests evenly spaced in range 0-255)
-            or an array of actual values to test
-            testGamma : (=1.0) the gamma value at which to test
-        autoMode : (='auto'). If 'auto' the program will present the screen
+    
+        photometer : a photometer object 
+            e.g. a :class:`~psychopy.hardware.pr.PR65` or
+            :class:`~psychopy.hardware.minolta.LS100` from hardware.findPhotometer()
+        
+        lumLevels : (default=8) 
+            array of values to test or single value for n evenly spaced test values
+        
+        gamma : (default=1.0) the gamma value at which to test
+        
+        autoMode : 'auto' or 'semi'(='auto')
+            
+            If 'auto' the program will present the screen
             and automatically take a measurement before moving on.
+            
             If set to 'semi' the program will wait for a keypress before
             moving on but will not attempt to make a measurement (use this
-            to make a measurement with your own device). Any other text will
-            simply move on without pausing on each screen (use this to see
-            that the visual system is performing as expected).
-
+            to make a measurement with your own device). 
+            
+            Any other value will simply move on without pausing on each screen (use this to see
+            that the display is performing as expected).
+    
     """
     import psychopy.event, psychopy.visual
-    #setup pr650
-    if isinstance(photometer, Photometer):
-        pr650=photometer
-    else:
-        pr650 = Photometer(photometer)
+    from psychopy import core
 
-    if pr650!=None:
-        havePR650 = 1
-    else: havePR650 = 0
+    if photometer==None:
+        havePhotom = False
+    elif not hasattr(photometer, 'getLum'):
+        log.error("photometer argument to monitors.getLumSeries should be a type of photometer "+\
+            "object, not a %s" %type(photometer))
+        return None
+    else: havePhotom = True
 
     if useBits:
         #all gamma transforms will occur in calling the Bits++ LUT
@@ -893,38 +770,46 @@ def getLumSeriesPR650(lumLevels=8,
 
     if gamma==1:
         initRGB= 0.5**(1/2.0)*2-1
-    else: initRGB=0.0
+    else: initRGB=0.8
     #setup screen and "stimuli"
-    myWin = psychopy.visual.Window(fullscr = 0, rgb=initRGB, size=winSize,
-        gamma=gamma,units='norm',monitor=monitor,allowGUI=False,
+    myWin = psychopy.visual.Window(fullscr = 0, size=winSize,
+        gamma=gamma,units='norm',monitor=monitor,allowGUI=True,winType='pyglet',
         bitsMode=bitsMode)
-
-    instructions="Point the PR650 at the central bar. Hit a key when ready (or wait 30s)"
+    instructions="Point the photometer at the central bar. Hit a key when ready (or wait 30s)"
     message = psychopy.visual.TextStim(myWin, text = instructions,
-        pos=(0,-0.8), rgb=-1.0)
-    message.draw()
-
+        pos=(0,-0.95), rgb=[1,-1,-1])    
     noise = numpy.random.rand(512,512).round()*2-1
     backPatch = psychopy.visual.PatchStim(myWin, tex=noise, size=2, units='norm',
         sf=[winSize[0]/512.0, winSize[1]/512.0])
     testPatch = psychopy.visual.PatchStim(myWin,
         tex='sqr',
         size=stimSize,
-        rgb=0.3,
+        rgb=initRGB,
         units='norm')
-    backPatch.draw()
-    testPatch.draw()
-
-    myWin.flip()
-
+   
     #stay like this until key press (or 30secs has passed)
-    psychopy.event.waitKeys(30)
-
+    waitClock=core.Clock()
+    tRemain=30
+    while tRemain>0:
+        tRemain = 30-waitClock.getTime()
+        instructions="Point the photometer at the central white bar. Hit a key when ready (or wait %iss)" %tRemain
+        backPatch.draw()
+        testPatch.draw()
+        message.setText(instructions)
+        message.draw()  
+        myWin.flip()
+        if len(psychopy.event.getKeys()):
+            break#we got a keypress so move on
+    
+    #
+    if photometer.type=='LS100':#LS100 likes to take at least one bright measurement
+        junk=photometer.getLum()
+    
     #what are the test values of luminance
     if (type(lumLevels) is int) or (type(lumLevels) is float):
         toTest= DACrange(lumLevels)
     else: toTest= numpy.asarray(lumLevels)
-
+    
     if allGuns: guns=[0,1,2,3]#gun=0 is the white luminance measure
     else: allGuns=[0]
     lumsList = numpy.zeros((len(guns),len(toTest)), 'd') #this will hoold the measured luminance values
@@ -940,37 +825,55 @@ def getLumSeriesPR650(lumLevels=8,
                 rgb[gun-1]=lum
             else:
                 rgb = [lum,lum,lum]
-
+            
             backPatch.draw()
-            testPatch.setRGB(rgb)
+            testPatch.setColor(rgb)
             testPatch.draw()
-
             myWin.flip()
-            time.sleep(0.5)#allowing the screen to settle (no good reason!)
+            
+            time.sleep(0.2)#allowing the screen to settle (no good reason!)
             #check for quit request
             for thisKey in psychopy.event.getKeys():
                 if thisKey in ['q', 'Q']:
                     myWin.close()
                     return numpy.array([])
             #take measurement
-            if havePR650 and autoMode=='auto':
-                pr650.measure()
-                print "At DAC value %i\t: %.2fcd/m^2" % (DACval, pr650.lastLum)
+            if havePhotom and autoMode=='auto':
+                actualLum = photometer.getLum()
+                print "At DAC value %i\t: %.2fcd/m^2" % (DACval, actualLum)
                 if lum==-1 or not allGuns:
                     #if the screen is black set all guns to this lum value!
-                    lumsList[:,valN] = pr650.lastLum
+                    lumsList[:,valN] = actualLum
                 else:
                     #otherwise just this gun
-                    lumsList[gun,valN] =  pr650.lastLum
+                    lumsList[gun,valN] =  actualLum
             elif autoMode=='semi':
                 print "At DAC value %i" % DACval
                 psychopy.event.waitKeys()
 
     myWin.close() #we're done with the visual stimuli
-    if havePR650:       return lumsList
+    if havePhotom: return lumsList
     else: return numpy.array([])
 
 
+def getLumSeriesPR650(lumLevels=8,
+    winSize=(800,600),
+    monitor=None,
+    gamma=1.0,
+    allGuns = True,
+    useBits=False,
+    autoMode='auto',
+    stimSize = 0.3,
+    photometer='COM1'):
+    """DEPRECATED (since v1.60.01): Use :class:`pscyhopy.monitors.getLumSeries()` instead"""
+    
+    log.warning("DEPRECATED (since v1.60.01): Use monitors.getLumSeries() instead")
+    val= getLumSeries(lumLevels,
+        winSize,monitor,
+        gamma,allGuns, useBits,
+        autoMode,stimSize,photometer)
+    return val
+
 def getRGBspectra(stimSize=0.3, winSize=(800,600), photometer='COM1'):
     """
     usage:
@@ -1007,7 +910,7 @@ def getRGBspectra(stimSize=0.3, winSize=(800,600), photometer='COM1'):
     spectra=[]
     for thisColor in [[1,-1,-1], [-1,1,-1], [-1,-1,1]]:
         #update stimulus
-        testPatch.setRGB(thisColor)
+        testPatch.setColor(thisColor)
         testPatch.draw()
         myWin.flip()
         #make measurement
